#!/bin/bash

# Created on: 2017.2.27

###################################################################################################################################################################
#Example:

#install master node:
#sh Setup.sh -type 1 -ip 192.168.158.136 -vip 192.168.158.138 -systemfile 16 -datafile 32 -treefile 32 -kvfile 32 -tablefile 64 -indexfile 64 -cachesize 64 -logs 4

#install slave node:
#sh Setup.sh -type 2 -ip 192.168.158.136 -masterip 192.168.158.139 -user root -password 123
###################################################################################################################################################################

#Set this to be real install patch.
INSTALL_PATH="/"

INSTALL_ROOT_DIR="Vernox"
INSTALL_LOCATION=${INSTALL_PATH}${INSTALL_ROOT_DIR}

SERVICE_LOCK="/var/lock/subsys/vernoxservice"
STORAGE_LOCK="/var/lock/subsys/vernoxstorage"

VERNOX_SYSCTL_CONF="/etc/sysctl.d/78-vernox.conf"

DBFILE=${INSTALL_LOCATION}"/Data/Vernox"

INSTALL_PARAMS=''

isSystemd=true

if [ $# != 0 ]
then
	INSTALL_PARAMS=$*
fi

check_systemd()
{
	tmp=`systemctl --version` 1>&2 2>/dev/null
	if [ $? == 127 ]
	then
		isSystemd=false
	fi
}

setup_traps()
{
	trap "cleanup" INT HUP QUIT FPE KILL TERM
}

cleanup()
{
	if [ $isSystemd == true ]
	then
		systemctl stop vernoxservice.service
		systemctl stop vernoxstorage.service
		systemctl disable vernoxservice.service 1>&2 2>/dev/null
		systemctl disable vernoxstorage.service 1>&2 2>/dev/null
		rm -rf /etc/init.d/vernoxservice
		rm -rf /etc/init.d/vernoxstorage
		rm -rf /etc/systemd/system/vernoxservice.service
		rm -rf /etc/systemd/system/vernoxstorage.service
	else
		service vernoxservice stop
		service vernoxstorage stop
		chkconfig vernoxservice off
		chkconfig vernoxstorage off
		chkconfig --del vernoxservice
		chkconfig --del vernoxstorage
		rm -rf /etc/init.d/vernoxservice
		rm -rf /etc/init.d/vernoxstorage
	fi
    
	if [ -e $DBFILE ]
	then
		rm -rf $DBFILE
	fi
    
	ipcrm -M 0x53425452
	ipcrm -M 0x53424C4F
	ipcrm -M 0x5342534D
	ipcrm -M 0x5342524D
	ipcrm -M 0x53424601
	ipcrm -M 0x53424602
	ipcrm -M 0x53424603
	ipcrm -M 0x53424604
	ipcrm -M 0x53424342
        exit 1
}

check_installed()
{
	SERVICE_INSTASLLED=false
	if [ $isSystemd == true ]
	then
		if [ -e /etc/init.d/vernoxservice -a -e /etc/init.d/vernoxstorage -a -e /etc/systemd/system/vernoxservice.service -a -e /etc/systemd/system/vernoxstorage.service ]
		then
			SERVICE_INSTASLLED=true
		fi
	else
		if [ "$(chkconfig --list | grep vernoxservice)""X" != "X" -a "$(chkconfig --list | grep vernoxstorage)""X" != "X" ]
		then
			SERVICE_INSTASLLED=true
		fi
	fi

	if [ ${SERVICE_INSTASLLED} == true ]
	then
		echo "Vernox had been installed previously, Can not install again."
		exit
	fi
}

# $1 = protocol:tcp/udp/...
# $2 = port
#example : allow_port_of_protocol tcp 8000
allow_port_of_protocol()
{
	if [ "X" == "$(command -v firewall-cmd)X" ]
	then
		service iptables status 1>/dev/null 2>&1
		if [ $? -eq 0 ]
		then
			iptables -I INPUT -p $1 --dport $2 -j ACCEPT 1>/dev/null 2>&1
			/etc/init.d/iptables save 1>/dev/null 2>&1
			service iptables restart 1>/dev/null 2>&1
		fi
	else
		if [ "noX" == "$(firewall-cmd --query-port=$2/$1 2>/dev/null)X" ]
		then
			firewall-cmd --add-port=$2/$1 1>/dev/null 2>&1
			firewall-cmd --add-port=$2/$1 --permanent 1>/dev/null 2>&1
			firewall-cmd --reload 1>/dev/null 2>&1
		fi
	fi
}

setup_firewall()
{
	allow_port_of_protocol tcp 8000
	allow_port_of_protocol tcp 3306
}

setup_system()
{
	#Avoid tcp TIME_WAIT status.
	if [ -e ${VERNOX_SYSCTL_CONF} ]
	then
		rm -rf ${VERNOX_SYSCTL_CONF}
	fi
    
	echo "net.ipv4.tcp_tw_recycle = 1" >> ${VERNOX_SYSCTL_CONF}
	echo "net.ipv4.tcp_tw_reuse = 1" >> ${VERNOX_SYSCTL_CONF}
}

install_path_set()
{
	echo "Vernox will be installed in the root path"
	echo "Input the path you want to change it, enter to pass"

	read CUSTOM_PATH

	if [ ${#CUSTOM_PATH} -ne 0 ]
	then
		if [ ${CUSTOM_PATH:0:1} != '/' ]
		then
			echo "please input absolute path"
			exit
		fi

		if [ ${CUSTOM_PATH:${#CUSTOM_PATH}-1:1} == '/' ]
		then
			CUSTOM_PATH=${CUSTOM_PATH:0:${#CUSTOM_PATH}-1}
		fi

		if [ ! -d ${CUSTOM_PATH} ]
		then
			mkdir -p ${CUSTOM_PATH} 2>/dev/null
			ret=$?
			if [ ${ret} -ne 0 ]
			then
				echo "mkdir return error:"${ret}
				exit
			fi
		fi

		INSTALL_PATH=${CUSTOM_PATH}"/"
		INSTALL_LOCATION=${INSTALL_PATH}${INSTALL_ROOT_DIR}
		DBFILE=${INSTALL_LOCATION}"/Data/Vernox"
	fi

	LINE=`sed -n '/^VENOX_INSTALL_PATH=/=' './Script/vernoxservice'`
	sed -i "${LINE}s#.*#"VENOX_INSTALL_PATH=\"${INSTALL_PATH}\""#" ./Script/vernoxservice

	LINE=`sed -n '/^VENOX_INSTALL_PATH=/=' './Script/vernoxstorage'`
	sed -i "${LINE}s#.*#"VENOX_INSTALL_PATH=\"${INSTALL_PATH}\""#" ./Script/vernoxstorage

	LINE=`sed -n '/^INSTALL_PATH=/=' './Uninstall.sh'`
	sed -i "${LINE}s#.*#"INSTALL_PATH=\"${INSTALL_PATH}\""#" ./Uninstall.sh
}

install_version()
{
	if [[ ! -e ${INSTALL_LOCATION} || $(cd ${INSTALL_LOCATION} && pwd) != $(pwd) ]]
	then
		rm -rf ${INSTALL_LOCATION}

		mkdir ${INSTALL_LOCATION}
		mkdir ${INSTALL_LOCATION}"/Bin"
		mkdir ${INSTALL_LOCATION}"/Data"
		mkdir ${INSTALL_LOCATION}"/Lib"

		\cp ./Bin/* ${INSTALL_LOCATION}"/Bin"
		\cp ./Setup.sh ${INSTALL_LOCATION}
		\cp ./Uninstall.sh ${INSTALL_LOCATION}
	else
		if [ ! -x ${INSTALL_LOCATION}"/Data" ]
		then
			mkdir ${INSTALL_LOCATION}"/Data"
		fi

		if [ ! -x ${INSTALL_LOCATION}"/Lib" ]
		then
			mkdir ${INSTALL_LOCATION}"/Lib"
		fi
	fi

	chmod +x ${INSTALL_LOCATION}"/Bin/Setup" ${INSTALL_LOCATION}"/Bin/Vernox" ${INSTALL_LOCATION}"/Bin/VernoxStorage" ${INSTALL_LOCATION}"/Bin/VernoxService"
}

install_services()
{
	if [ $isSystemd == true ]
	then
		\cp ./Script/vernoxservice.service /etc/systemd/system/
		\cp ./Script/vernoxstorage.service /etc/systemd/system/
		\cp ./Script/vernoxstorage /etc/init.d/
		\cp ./Script/vernoxservice /etc/init.d/

		chmod +x /etc/init.d/vernoxstorage /etc/init.d/vernoxservice

		systemctl enable vernoxservice.service 1>&2 2>/dev/null
		systemctl enable vernoxstorage.service 1>&2 2>/dev/null
		systemctl daemon-reload
	else
		\cp ./Script/vernoxstorage /etc/init.d/
		\cp ./Script/vernoxservice /etc/init.d/

		chmod +x /etc/init.d/vernoxstorage /etc/init.d/vernoxservice

		chkconfig --add vernoxstorage
		chkconfig --add vernoxservice
		chkconfig vernoxstorage on
		chkconfig vernoxservice on
	fi
}

setup_version()
{
	${INSTALL_LOCATION}"/Bin/Setup" ${INSTALL_PARAMS}
}

#setup sequences
setup_traps
check_systemd
check_installed
setup_firewall
setup_system
install_path_set
install_version
install_services
setup_version
