#!/bin/bash

# Created on: 2019.11.11

VERNOX_SYSCTL_CONF="/etc/sysctl.d/78-vernox.conf"

isSystemd=true

setup_traps()
{
	trap "cleanup" INT HUP QUIT FPE KILL TERM
}

check_systemd()
{
	tmp=`systemctl --version` 1>&2 2>/dev/null
	if [ $? == 127 ]
	then
		isSystemd=false
	fi
}

stop_pre_service()
{
	if [ $isSystemd == true ]
	then
		systemctl stop vernoxservice.service 1>&2 2>/dev/null
		systemctl stop vernoxstorage.service 1>&2 2>/dev/null
		systemctl disable vernoxservice.service 1>&2 2>/dev/null
		systemctl disable vernoxstorage.service 1>&2 2>/dev/null
		rm -rf /etc/init.d/vernoxservice 1>&2 2>/dev/null
		rm -rf /etc/init.d/vernoxstorage 1>&2 2>/dev/null
		rm -rf /etc/systemd/system/vernoxservice.service 1>&2 2>/dev/null
		rm -rf /etc/systemd/system/vernoxstorage.service 1>&2 2>/dev/null
	else
		service vernoxservice stop 1>&2 2>/dev/null
		service vernoxstorage stop 1>&2 2>/dev/null
		chkconfig vernoxservice off 1>&2 2>/dev/null
		chkconfig vernoxstorage off 1>&2 2>/dev/null
		chkconfig --del vernoxservice 1>&2 2>/dev/null
		chkconfig --del vernoxstorage 1>&2 2>/dev/null
		rm -rf /etc/init.d/vernoxservice 1>&2 2>/dev/null
		rm -rf /etc/init.d/vernoxstorage 1>&2 2>/dev/null
	fi
}

allow_port_of_protocol()
{
	if [ "X" == "$(command -v firewall-cmd)X" ]
	then
		service iptables status 1>/dev/null 2>&1
		if [ $? -eq 0 ]
		then
			iptables -I INPUT -p $1 --dport $2 -j ACCEPT 1>/dev/null 2>&1
			/etc/init.d/iptables save 1>/dev/null 2>&1
			service iptables restart 1>/dev/null 2>&1
		fi
	else
		if [ "noX" == "$(firewall-cmd --query-port=$2/$1 2>/dev/null)X" ]
		then
			firewall-cmd --add-port=$2/$1 1>/dev/null 2>&1
			firewall-cmd --add-port=$2/$1 --permanent 1>/dev/null 2>&1
			firewall-cmd --reload 1>/dev/null 2>&1
		fi
	fi
}

setup_firewall()
{
	vernox_port=$(sed -n "/^listenport=/p" Bin/VernoxConfig.ini | awk -F= '{print $2}')
	if [ ${vernox_port}"X" == "X" ]
	then
		vernox_port=8000
	fi
	
	allow_port_of_protocol tcp ${vernox_port}
	
	mysql_port=$(sed -n "/^mysqlport=/p" Bin/VernoxConfig.ini | awk -F= '{print $2}')
	if [ ${mysql_port}"X" == "X" ]
	then
		mysql_port=3306
	fi
	
	allow_port_of_protocol tcp ${mysql_port}
}

setup_system()
{
	#Avoid tcp TIME_WAIT status.
	if [ -e ${VERNOX_SYSCTL_CONF} ]
	then
		rm -rf ${VERNOX_SYSCTL_CONF}
	fi

	echo "net.ipv4.tcp_tw_recycle = 1" >> ${VERNOX_SYSCTL_CONF}
	echo "net.ipv4.tcp_tw_reuse = 1" >> ${VERNOX_SYSCTL_CONF}
}

install_path_set()
{
	INSTALL_PATH=$(cd "$(dirname "$0")";pwd)
	INSTALL_PATH=${INSTALL_PATH:0:$((${#INSTALL_PATH}-6))}

	LINE=`sed -n '/^VENOX_INSTALL_PATH=/=' './Script/vernoxservice'`
	sed -i "${LINE}s#.*#"VENOX_INSTALL_PATH=\"${INSTALL_PATH}\""#" ./Script/vernoxservice

	LINE=`sed -n '/^VENOX_INSTALL_PATH=/=' './Script/vernoxstorage'`
	sed -i "${LINE}s#.*#"VENOX_INSTALL_PATH=\"${INSTALL_PATH}\""#" ./Script/vernoxstorage

	LINE=`sed -n '/^INSTALL_PATH=/=' './Uninstall.sh'`
	sed -i "${LINE}s#.*#"INSTALL_PATH=\"${INSTALL_PATH}\""#" ./Uninstall.sh
}

install_services()
{
	if [ $isSystemd == true ]
	then
		\cp ./Script/vernoxservice.service /etc/systemd/system/
		\cp ./Script/vernoxstorage.service /etc/systemd/system/
		\cp ./Script/vernoxstorage /etc/init.d/
		\cp ./Script/vernoxservice /etc/init.d/

		chmod +x /etc/init.d/vernoxstorage /etc/init.d/vernoxservice

		chmod 644 /etc/systemd/system/vernoxstorage.service
		chmod 644 /etc/systemd/system/vernoxservice.service

		systemctl enable vernoxservice.service 1>&2 2>/dev/null
		systemctl enable vernoxstorage.service 1>&2 2>/dev/null
		systemctl daemon-reload
	else
		\cp ./Script/vernoxstorage /etc/init.d/
		\cp ./Script/vernoxservice /etc/init.d/

		chmod +x /etc/init.d/vernoxstorage /etc/init.d/vernoxservice

		chkconfig --add vernoxstorage
		chkconfig --add vernoxservice
		chkconfig vernoxstorage on
		chkconfig vernoxservice on
	fi
}

modify_config_file()
{
	sed -i "/install=/d"  Bin/VernoxConfig.ini

	sed -i "/^\[Config\]/i\install=1" Bin/VernoxConfig.ini
}

startup_services()
{
	if [ $isSystemd == true ]
	then
		systemctl start vernoxstorage.service 1>&2 2>/dev/null
		systemctl start vernoxservice.service 1>&2 2>/dev/null
	else
		service vernoxstorage start 1>&2 2>/dev/null
		service vernoxservice start 1>&2 2>/dev/null
	fi
}

echo_ok()
{
	echo "Recovery ok"
}

#recovery sequences
setup_traps
check_systemd
stop_pre_service
setup_firewall
setup_system
install_path_set
install_services
modify_config_file
startup_services
echo_ok
