#!/bin/bash

# Created on: 2019.12.12

isSystemd=true

#the path of vernox backup, set default to "/infinityfs1", can be modified according to requirement.
BACKUP_PATH=""

INSTALL_PATH=""

setup_traps()
{
	trap "cleanup" INT HUP QUIT FPE KILL TERM
}

check_systemd()
{
	tmp=`systemctl --version` 1>&2 2>/dev/null
	if [ $? == 127 ]
	then
		isSystemd=false
	fi
}

check_install_path()
{
	service_instaslled=false

	if [ $isSystemd == true ]
	then
		if [ -e /etc/init.d/vernoxservice -a -e /etc/init.d/vernoxstorage -a -e /etc/systemd/system/vernoxservice.service -a -e /etc/systemd/system/vernoxstorage.service ]
		then
			service_instaslled=true
		fi
	else
		if [ "$(chkconfig --list | grep vernoxservice)""X" != "X" -a "$(chkconfig --list | grep vernoxstorage)""X" != "X" ]
		then
			service_instaslled=true
		fi
	fi
    
	if [ ${service_instaslled} == false ]
	then
		echo "Vernox had not been installed previously, Can not upgrade."
		exit
	fi
    
    INSTALL_PATH=$(sed -n "/^VENOX_INSTALL_PATH/p" /etc/init.d/vernoxservice | awk -F= '{print $2}' | sed 's/\"//g')
    
    if [ ${INSTALL_PATH:${#INSTALL_PATH}-1:1} == '/' ]
    then
        INSTALL_PATH=${INSTALL_PATH:0:${#INSTALL_PATH}-1}
    fi
    
    if [ 'X' == $BACKUP_PATH'X' ]
    then
        BACKUP_PATH=$INSTALL_PATH
    fi
    
    if [ 'X' != $BACKUP_PATH'X' ]
    then
        if [ ${BACKUP_PATH:${#BACKUP_PATH}-1:1} == '/' ]
        then
            BACKUP_PATH=${BACKUP_PATH:0:${#BACKUP_PATH}-1}
        fi    
    fi
}

stop_pre_service()
{
	if [ $isSystemd == true ]
	then
		systemctl stop vernoxservice.service 1>&2 2>/dev/null
		systemctl stop vernoxstorage.service 1>&2 2>/dev/null
	else
		service vernoxservice stop 1>&2 2>/dev/null
		service vernoxstorage stop 1>&2 2>/dev/null
	fi
}

backup_data()
{
    if [ -d $BACKUP_PATH"/Vernox/Data_bak" ]
    then
        rm -rf $BACKUP_PATH"/Vernox/Data_bak"
    fi

    mkdir -p $BACKUP_PATH"/Vernox/Data_bak"
    
    for dbf_file in $INSTALL_PATH"/Vernox/Data/Vernox/"*.dbf;
    do 
        cp ${dbf_file} $BACKUP_PATH"/Vernox/Data_bak/"
    done
}

backup_services()
{
    if [ -d $BACKUP_PATH"/Vernox/Bin_bak" ]
    then
        rm -rf $BACKUP_PATH"/Vernox/Bin_bak"
    fi

    mkdir -p $BACKUP_PATH"/Vernox/Bin_bak"
    
    cp $INSTALL_PATH"/Vernox/Bin/Setup" $BACKUP_PATH"/Vernox/Bin_bak/" 1>&2 2>/dev/null
    cp $INSTALL_PATH"/Vernox/Bin/Vernox" $BACKUP_PATH"/Vernox/Bin_bak/" 1>&2 2>/dev/null
    cp $INSTALL_PATH"/Vernox/Bin/VernoxService" $BACKUP_PATH"/Vernox/Bin_bak/" 1>&2 2>/dev/null
    cp $INSTALL_PATH"/Vernox/Bin/VernoxStorage" $BACKUP_PATH"/Vernox/Bin_bak/" 1>&2 2>/dev/null
    cp $INSTALL_PATH"/Vernox/Bin/VernoxInterface.so" $BACKUP_PATH"/Vernox/Bin_bak/" 1>&2 2>/dev/null
}

upgrade_services()
{
    \cp "./Bin/Setup" $INSTALL_PATH"/Vernox/Bin/" 1>&2 2>/dev/null
    \cp "./Bin/Vernox" $INSTALL_PATH"/Vernox/Bin/" 1>&2 2>/dev/null
    \cp "./Bin/VernoxService" $INSTALL_PATH"/Vernox/Bin/" 1>&2 2>/dev/null
    \cp "./Bin/VernoxStorage" $INSTALL_PATH"/Vernox/Bin/" 1>&2 2>/dev/null
    \cp "./Bin/VernoxInterface.so" $INSTALL_PATH"/Vernox/Bin/" 1>&2 2>/dev/null
    
    chmod +x $INSTALL_PATH"/Bin/Setup" 1>&2 2>/dev/null
    chmod +x $INSTALL_PATH"/Bin/Vernox" 1>&2 2>/dev/null
    chmod +x $INSTALL_PATH"/Bin/VernoxStorage" 1>&2 2>/dev/null
    chmod +x $INSTALL_PATH"/Bin/VernoxService" 1>&2 2>/dev/null
}

startup_services()
{
	if [ $isSystemd == true ]
	then
		systemctl start vernoxstorage.service 1>&2 2>/dev/null
		systemctl start vernoxservice.service 1>&2 2>/dev/null
	else
		service vernoxstorage start 1>&2 2>/dev/null
		service vernoxservice start 1>&2 2>/dev/null
	fi
}

echo_ok()
{
	echo "Upgrade ok"
}

#recovery sequences
setup_traps
check_systemd
check_install_path
stop_pre_service
backup_data
backup_services
upgrade_services
startup_services
echo_ok
